@@ -86,6 +86,7 @@ private:
     bool m_showVersion;
     std::string m_showOptions;
     bool m_showJSON;
+    std::string m_log;
 };
 
 
@@ -206,7 +207,8 @@ void App::outputOptions(std::string const& stageName, std::ostream& strm)
 
     if (!m_showJSON)
     {
-        strm  << stageName << " -- " << PluginManager::link(stageName) << std::endl;
+        strm  << stageName << " -- " << PluginManager::link(stageName) <<
+            std::endl;
         strm  << headline << std::endl;
 
         args.dump2(strm , 2, 6, headline.size());
@@ -279,6 +281,8 @@ void App::addArgs(ProgramArgs& args)
     args.add("version", "Show program version", m_showVersion);
     args.add("options", "Show options for specified driver (or 'all')",
         m_showOptions);
+    args.add("log", "Log filename (accepts stderr, stdout, stdlog, devnull"
+        " as special cases)", m_log, "stderr");
     Arg& json = args.add("showjson", "List options or drivers as JSON output",
         m_showJSON);
     json.setHidden();
@@ -344,6 +348,7 @@ int App::execute(StringList& cmdArgs, LogPtr& log)
         return -1;
     }
 
+    log.reset(new Log("PDAL", m_log));
     if (m_logLevel != LogLevel::None)
         log->setLevel(m_logLevel);
     else if (m_debug)
