@@ -37,6 +37,7 @@
 #include <iostream>
 
 #include <pdal/Compression.hpp>
+#include <pdal/DimUtil.hpp>
 #include <pdal/PDALUtils.hpp>
 #include <pdal/PointView.hpp>
 #include <pdal/util/Algorithm.hpp>
@@ -64,7 +65,8 @@ CREATE_STATIC_PLUGIN(1, 0, LasWriter, Writer, s_info)
 
 std::string LasWriter::getName() const { return s_info.name; }
 
-LasWriter::LasWriter() : m_ostream(NULL), m_compression(LasCompression::None)
+LasWriter::LasWriter() : m_ostream(NULL), m_compression(LasCompression::None),
+    m_srsCnt(0)
 {}
 
 
@@ -104,12 +106,15 @@ void LasWriter::addArgs(ProgramArgs& args)
         decltype(m_creationDoy)(doy));
     args.add("creation_year", "Creation year", m_creationYear,
         decltype(m_creationYear)(year));
+    args.add("pdal_metadata", "Write PDAL metadata as VLR?", m_writePDALMetadata,
+        decltype(m_writePDALMetadata)(false));
     args.add("scale_x", "X scale factor", m_scaleX, decltype(m_scaleX)(".01"));
     args.add("scale_y", "Y scale factor", m_scaleY, decltype(m_scaleY)(".01"));
     args.add("scale_z", "Z scale factor", m_scaleZ, decltype(m_scaleZ)(".01"));
     args.add("offset_x", "X offset", m_offsetX);
     args.add("offset_y", "Y offset", m_offsetY);
     args.add("offset_z", "Z offset", m_offsetZ);
+    args.add("vlrs", "List of VLRs to set", m_userVLRs);
 }
 
 void LasWriter::initialize()
@@ -125,11 +130,28 @@ void LasWriter::initialize()
         m_lasHeader.setCompressed(true);
 #if !defined(PDAL_HAVE_LASZIP) && !defined(PDAL_HAVE_LAZPERF)
     if (m_compression != LasCompression::None)
-        throw pdal_error("Can't write LAZ output.  "
-            "PDAL not built with LASzip or LAZperf.");
+        throwError("Can't write LAZ output.  PDAL not built with "
+            "LASzip or LAZperf.");
 #endif
-    m_extraDims = LasUtils::parse(m_extraDimSpec);
+    try
+    {
+        m_extraDims = LasUtils::parse(m_extraDimSpec);
+    }
+    catch (const LasUtils::error& err)
+    {
+        throwError(err.what());
+    }
     fillForwardList();
+    collectUserVLRs();
+}
+
+
+void LasWriter::spatialReferenceChanged(const SpatialReference&)
+{
+    if (++m_srsCnt > 1)
+        log()->get(LogLevel::Error) << getName() <<
+            ": Attempting to write '" << m_filename << "' with multiple "
+            "point spatial references." << std::endl;
 }
 
 
@@ -139,6 +161,10 @@ void LasWriter::prepared(PointTableRef table)
 
     PointLayoutPtr layout = table.layout();
 
+    // Make sure the dataformatID is set so that we can get the proper
+    // dimensions being written as part of the standard LAS record.
+    fillHeader();
+
     // If we've asked for all dimensions, add to extraDims all dimensions
     // in the layout that aren't already destined for LAS output.
     if (m_extraDims.size() == 1 && m_extraDims[0].m_name == "all")
@@ -159,13 +185,45 @@ void LasWriter::prepared(PointTableRef table)
     {
         dim.m_dimType.m_id = table.layout()->findDim(dim.m_name);
         if (dim.m_dimType.m_id == Dimension::Id::Unknown)
-        {
-            std::ostringstream oss;
-            oss << "Dimension '" << dim.m_name << "' specified in "
-                "'extra_dim' option not found.";
-            throw pdal_error(oss.str());
-        }
+            throwError("Dimension '" + dim.m_name + "' specified in "
+                "'extra_dim' option not found.");
         m_extraByteLen += Dimension::size(dim.m_dimType.m_type);
+        log()->get(LogLevel::Info) << getName() << ": Writing dimension " <<
+            dim.m_name <<
+            "(" << Dimension::interpretationName(dim.m_dimType.m_type) <<
+            ") " << " to LAS extra bytes." << std::endl;
+    }
+}
+
+
+// Capture user-specified VLRs
+void LasWriter::collectUserVLRs()
+{
+
+    for (const auto& v : m_userVLRs)
+    {
+        uint16_t recordId(1);
+        std::string userId("");
+        std::string description("");
+        std::string b64data("");
+        std::string user("");
+        if (! v.isMember("user_id"))
+            throw pdal_error("VLR must contain a 'user_id'!");
+        userId = v["user_id"].asString();
+
+        if (!v.isMember("data"))
+            throw pdal_error("VLR must contain a base64-encoded 'data' member");
+        b64data = v["data"].asString();
+
+        if (v.isMember("record_id"))
+            recordId = v["record_id"].asUInt64();
+
+        if (v.isMember("description"))
+            description = v["description"].asString();
+
+        std::vector<uint8_t> data = Utils::base64_decode(b64data);
+        addVlr(userId, recordId, description, data);
+
     }
 }
 
@@ -211,13 +269,8 @@ void LasWriter::fillForwardList()
         else if (Utils::contains(all, name))
             m_forwards.insert(name);
         else
-        {
-            std::ostringstream oss;
-
-            oss << "Error in 'forward' option.  Unknown field for "
-                "forwarding: '" << name << "'.";
-            throw pdal_error(oss.str());
-        }
+            throwError("Error in 'forward' option.  Unknown field for "
+                "forwarding: '" + name + "'.");
     }
 }
 
@@ -225,6 +278,9 @@ void LasWriter::fillForwardList()
 void LasWriter::readyTable(PointTableRef table)
 {
     m_forwardMetadata = table.privateMetadata("lasforward");
+    MetadataNode m = table.metadata();
+    if(m_writePDALMetadata)
+        setPDALVLRs(m);
     setExtraBytesVlr();
 }
 
@@ -234,13 +290,7 @@ void LasWriter::readyFile(const std::string& filename,
 {
     std::ostream *out = Utils::createFile(filename, true);
     if (!out)
-    {
-        std::stringstream out;
-
-        out << "writers.las couldn't open file '" << filename <<
-            "' for output.";
-        throw pdal_error(out.str());
-    }
+        throwError("Couldn't open file '" + filename + "' for output.");
     m_curFilename = filename;
     m_error.setFilename(filename);
     Utils::writeProgress(m_progressFd, "READYFILE", filename);
@@ -250,6 +300,9 @@ void LasWriter::readyFile(const std::string& filename,
 
 void LasWriter::prepOutput(std::ostream *outStream, const SpatialReference& srs)
 {
+
+
+
     // Use stage SRS if provided.
     m_srs = getSpatialReference().empty() ? srs : getSpatialReference();
 
@@ -263,6 +316,7 @@ void LasWriter::prepOutput(std::ostream *outStream, const SpatialReference& srs)
     setVlrsFromSpatialRef();
     setVlrsFromMetadata(m_forwardMetadata);
 
+
     m_summaryData.reset(new LasSummaryData());
     m_ostream = outStream;
     if (m_lasHeader.compressed())
@@ -329,6 +383,26 @@ MetadataNode LasWriter::findVlrMetadata(MetadataNode node,
     return node.find(pred);
 }
 
+void LasWriter::setPDALVLRs(MetadataNode& forward)
+{
+    auto store = [this](std::string json, int recordId, std::string description)
+    {
+        std::vector<uint8_t> data;
+        data.resize(json.size());
+        std::copy(json.begin(), json.end(), data.begin());
+        addVlr("PDAL", recordId, description, data);
+    };
+
+    std::ostringstream ostr;
+    Utils::toJSON(forward, ostr);
+    std::string json = ostr.str();
+    store(ostr.str(), 12, "PDAL metadata");
+
+    ostr.str("");
+    PipelineWriter::writePipeline(this, ostr);
+    store(ostr.str(), 13, "PDAL pipeline");
+}
+
 
 /// Set VLRs from metadata for forwarded info.
 void LasWriter::setVlrsFromMetadata(MetadataNode& forward)
@@ -376,42 +450,27 @@ void LasWriter::setVlrsFromSpatialRef()
 
 void LasWriter::addGeotiffVlrs()
 {
-    GeotiffSupport geotiff;
-    geotiff.resetTags();
-
-    geotiff.setWkt(m_srs.getWKT());
-
-    addGeotiffVlr(geotiff, GEOTIFF_DIRECTORY_RECORD_ID,
-        "GeoTiff GeoKeyDirectoryTag");
-    addGeotiffVlr(geotiff, GEOTIFF_DOUBLES_RECORD_ID,
-        "GeoTiff GeoDoubleParamsTag");
-    addGeotiffVlr(geotiff, GEOTIFF_ASCII_RECORD_ID,
-        "GeoTiff GeoAsciiParamsTag");
-}
-
-
-/// Add a geotiff VLR from the information associated with the record ID.
-/// \param  geotiff - Geotiff support structure reference.
-/// \param  recordId - Record ID associated with the VLR/Geotiff ref.
-/// \param  description - Description to use with the VLR
-/// \return  Whether the VLR was added.
-void LasWriter::addGeotiffVlr(GeotiffSupport& geotiff, uint16_t recordId,
-    const std::string& description)
-{
-    void *data;
-    int count;
-
-    size_t size = geotiff.getKey(recordId, &count, &data);
-    if (size == 0)
-    {
-        log()->get(LogLevel::Warning) << getName() << ": Invalid spatial "
-            "reference for writing GeoTiff VLR." << std::endl;
+    if (m_srs.empty())
         return;
-    }
 
-    std::vector<uint8_t> buf(size);
-    memcpy(buf.data(), data, size);
-    addVlr(TRANSFORM_USER_ID, recordId, description, buf);
+    try
+    {
+        GeotiffTags tags(m_srs);
+
+        if (tags.directoryData().empty())
+            throwError("Invalid spatial reference for writing GeoTiff VLR.");
+
+        addVlr(TRANSFORM_USER_ID, GEOTIFF_DIRECTORY_RECORD_ID,
+                "GeoTiff GeoKeyDirectoryTag", tags.directoryData());
+        addVlr(TRANSFORM_USER_ID, GEOTIFF_DOUBLES_RECORD_ID,
+                "GeoTiff GeoDoubleParamsTag", tags.doublesData());
+        addVlr(TRANSFORM_USER_ID, GEOTIFF_ASCII_RECORD_ID,
+                "GeoTiff GeoAsciiParamsTag", tags.asciiData());
+    }
+    catch (GeotiffTags::error& err)
+    {
+        throwError(err.what());
+    }
 }
 
 
@@ -465,8 +524,15 @@ void LasWriter::addVlr(const std::string& userId, uint16_t recordId,
 {
     if (data.size() > LasVLR::MAX_DATA_SIZE)
     {
-        ExtLasVLR evlr(userId, recordId, description, data);
-        m_eVlrs.push_back(std::move(evlr));
+        if (m_lasHeader.versionAtLeast(1, 4))
+        {
+            ExtLasVLR evlr(userId, recordId, description, data);
+            m_eVlrs.push_back(std::move(evlr));
+        }
+        else
+            throwError("Can't write VLR with user ID/record ID = " +
+                userId + "/" + std::to_string(recordId) +
+                ".  The data size exceeds the maximum supported.");
     }
     else
     {
@@ -537,7 +603,14 @@ void LasWriter::fillHeader()
 {
     const uint16_t WKT_MASK = (1 << 4);
 
-    m_lasHeader.setScaling(m_scaling);
+    try
+    {
+        m_lasHeader.setScaling(m_scaling);
+    }
+    catch (const LasHeader::error& err)
+    {
+        throwError(err.what());
+    }
     m_lasHeader.setVlrCount(m_vlrs.size());
     m_lasHeader.setEVlrCount(m_eVlrs.size());
 
@@ -558,12 +631,8 @@ void LasWriter::fillHeader()
     m_lasHeader.setGlobalEncoding(globalEncoding);
 
     if (!m_lasHeader.pointFormatSupported())
-    {
-        std::ostringstream oss;
-        oss << "Unsupported LAS output point format: " <<
-            (int)m_lasHeader.pointFormat() << ".";
-        throw pdal_error(oss.str());
-    }
+        throwError("Unsupported LAS output point format: " +
+            Utils::toString((int)m_lasHeader.pointFormat()) + ".");
 }
 
 
@@ -579,8 +648,18 @@ void LasWriter::readyCompression()
 void LasWriter::readyLasZipCompression()
 {
 #ifdef PDAL_HAVE_LASZIP
-    m_zipPoint.reset(new LasZipPoint(m_lasHeader.pointFormat(),
-        m_lasHeader.pointLen()));
+    if (m_lasHeader.pointFormat() > 5)
+        throwError("LASzip doesn't currently support compression using LAS "
+            "1.4 point formats (dataformat_id > 5).");
+    try
+    {
+        m_zipPoint.reset(new LasZipPoint(m_lasHeader.pointFormat(),
+            m_lasHeader.pointLen()));
+    }
+    catch (const LasZipPoint::error& err)
+    {
+        throwError(err.what());
+    }
     m_zipper.reset(new LASzipper());
     // Note: this will make the VLR count in the header incorrect, but we
     // rewrite that bit in finishOutput() to fix it up.
@@ -594,7 +673,7 @@ void LasWriter::readyLazPerfCompression()
 {
 #ifdef PDAL_HAVE_LAZPERF
     if (m_lasHeader.versionAtLeast(1, 4))
-        throw pdal_error("Can't write version 1.4 output with LAZperf.");
+        throwError("Can't write version 1.4 output with LAZperf.");
 
     laszip::factory::record_schema schema;
     schema.push(laszip::factory::record_item::POINT10);
@@ -624,8 +703,7 @@ void LasWriter::openCompression()
         const char* err = m_zipper->get_error();
         if (err == NULL)
             err = "(unknown error)";
-        oss << "Error opening LASzipper: " << std::string(err);
-        throw pdal_error(oss.str());
+        throwError("Error opening LASzipper: " + std::string(err) + ".");
     }
 #endif
 }
@@ -694,8 +772,7 @@ void LasWriter::writeLasZipBuf(char *pos, size_t pointLen, point_count_t numPts)
             const char* err = m_zipper->get_error();
             if (err == NULL)
                 err = "(unknown error)";
-            oss << "Error writing point: " << std::string(err);
-            throw pdal_error(oss.str());
+            throwError("Error writing point: " + std::string(err) + ".");
         }
         pos += pointLen;
     }
@@ -754,16 +831,13 @@ bool LasWriter::fillPointBuf(PointRef& point, LeInserter& ostream)
 
     auto converter = [this](double d, Dimension::Id dim) -> int32_t
     {
-        int32_t i;
+        int32_t i(0);
 
         if (!Utils::numericCast(d, i))
-        {
-            std::ostringstream oss;
-            oss << "Unable to convert scaled value (" << d << ") to "
-                "int32 for dimension '" << Dimension::name(dim) <<
-                "' when writing LAS/LAZ file " << m_curFilename << ".";
-            throw pdal_error(oss.str());
-        }
+            throwError("Unable to convert scaled value (" +
+                Utils::toString(d) + ") to "
+                "int32 for dimension '" + Dimension::name(dim) +
+                "' when writing LAS/LAZ file " + m_curFilename + ".");
         return i;
     };
 
@@ -877,6 +951,7 @@ void LasWriter::doneFile()
 
 void LasWriter::finishOutput()
 {
+
     if (m_compression == LasCompression::LasZip)
         finishLasZipOutput();
     else if (m_compression == LasCompression::LazPerf)
@@ -888,6 +963,7 @@ void LasWriter::finishOutput()
 
     OLeStream out(m_ostream);
 
+    // addVlr prevents any eVlrs from being added before version 1.4.
     for (auto vi = m_eVlrs.begin(); vi != m_eVlrs.end(); ++vi)
     {
         ExtLasVLR evlr = *vi;
@@ -895,10 +971,24 @@ void LasWriter::finishOutput()
     }
 
     // Reset the offset/scale since it may have been auto-computed
-    m_lasHeader.setScaling(m_scaling);
+    try
+    {
+        m_lasHeader.setScaling(m_scaling);
+    }
+    catch (const LasHeader::error& err)
+    {
+        throwError(err.what());
+    }
 
     // The summary is calculated as points are written.
-    m_lasHeader.setSummary(*m_summaryData);
+    try
+    {
+        m_lasHeader.setSummary(*m_summaryData);
+    }
+    catch (const LasHeader::error& err)
+    {
+        throwError(err.what());
+    }
 
     out.seek(0);
     out << m_lasHeader;
