@@ -124,6 +124,8 @@ void TIndexKernel::validateSwitches(ProgramArgs& args)
         StringList invalidArgs;
         invalidArgs.push_back("a_srs");
         invalidArgs.push_back("src_srs_name");
+        invalidArgs.push_back("stdin");
+        invalidArgs.push_back("fast_boundary");
         for (auto arg : invalidArgs)
             if (args.set(arg))
             {
@@ -216,6 +218,10 @@ void TIndexKernel::createFile()
     else
         m_files = readSTDIN();
 
+    if (m_absPath)
+        for (auto& s : m_files)
+            s = FileUtils::toAbsolutePath(s);
+
     if (m_files.empty())
     {
         std::ostringstream out;
@@ -463,7 +469,10 @@ TIndexKernel::FileInfo TIndexKernel::getFileInfo(KernelFactory& factory,
     // Need to make sure options get set.
     Stage& reader = manager.makeReader(filename, "");
 
-    if (m_fastBoundary)
+    // If we aren't able to make a hexbin filter, we
+    // will just do a simple fast_boundary.
+    Stage* hexer = &manager.makeFilter("filters.hexbin", reader);
+    if (m_fastBoundary || !hexer)
     {
         QuickInfo qi = reader.preview();
 
@@ -482,11 +491,10 @@ TIndexKernel::FileInfo TIndexKernel::getFileInfo(KernelFactory& factory,
     }
     else
     {
-        Stage& hexer = manager.makeFilter("filters.hexbin", reader);
 
         PointTable table;
-        hexer.prepare(table);
-        PointViewSet set = hexer.execute(table);
+        hexer->prepare(table);
+        PointViewSet set = hexer->execute(table);
 
         MetadataNode m = table.metadata();
         m = m.findChild("filters.hexbin:boundary");
