@@ -34,6 +34,7 @@
 
 #include <pdal/EigenUtils.hpp>
 #include <pdal/GDALUtils.hpp>
+#include <pdal/KDIndex.hpp>
 #include <pdal/PointView.hpp>
 #include <pdal/SpatialReference.hpp>
 #include <pdal/util/Bounds.hpp>
@@ -42,6 +43,7 @@
 #include <Eigen/Dense>
 
 #include <cfloat>
+#include <numeric>
 #include <vector>
 
 namespace pdal
@@ -255,6 +257,44 @@ Eigen::MatrixXd createMaxMatrix(PointView& view, int rows, int cols,
     return ZImax;
 }
 
+Eigen::MatrixXd createMaxMatrix2(PointView& view, int rows, int cols,
+                                 double cell_size, BOX2D bounds)
+{
+    using namespace Dimension;
+    using namespace Eigen;
+
+    KD2Index kdi(view);
+    kdi.build();
+
+    MatrixXd ZImax(rows, cols);
+    ZImax.setConstant(std::numeric_limits<double>::quiet_NaN());
+
+    // for each grid center, search PointView for neighbors, and find max of those
+    for (int c = 0; c < cols; ++c)
+    {
+        double x = bounds.minx + (c + 0.5) * cell_size;
+
+        for (int r = 0; r < rows; ++r)
+        {
+            double y = bounds.miny + (r + 0.5) * cell_size;
+
+            auto neighbors = kdi.radius(x, y, cell_size * std::sqrt(2.0));
+            
+            double val(std::numeric_limits<double>::lowest());
+            for (auto const& n : neighbors)
+            {
+                double z(view.getFieldAs<double>(Id::Z, n));
+                if (z > val)
+                    val = z;
+            }
+            if (val > std::numeric_limits<double>::lowest())
+                ZImax(r, c) = val;
+        }
+    }
+
+    return ZImax;
+}
+
 Eigen::MatrixXd extendedLocalMinimum(PointView& view, int rows, int cols,
                                      double cell_size, BOX2D bounds)
 {
@@ -428,22 +468,72 @@ Eigen::MatrixXd matrixOpen(Eigen::MatrixXd data, int radius)
     return maxZ.block(radius, radius, data.rows(), data.cols());
 }
 
-Eigen::MatrixXd padMatrix(Eigen::MatrixXd d, int r)
+std::vector<double> dilateDiamond(std::vector<double> data, size_t rows, size_t cols, int iterations)
 {
-    using namespace Eigen;
+    std::vector<double> out(data.size(), std::numeric_limits<double>::lowest());
+    std::vector<size_t> idx(5);
+    
+    for (int iter = 0; iter < iterations; ++iter)
+    {
+        for (size_t col = 0; col < cols; ++col)
+        {
+            size_t index = col*rows;
+            for (size_t row = 0; row < rows; ++row)
+            {
+                size_t j = 0;
+                idx[j++] = index+row;
+                if (row > 0)
+                    idx[j++] = idx[0]-1;
+                if (row < rows-1)
+                    idx[j++] = idx[0]+1;
+                if (col > 0)
+                    idx[j++] = idx[0]-rows;
+                if (col < cols-1)
+                    idx[j++] = idx[0]+rows;
+                for (size_t i = 0; i < j; ++i)
+                {
+                    if (data[idx[i]] > out[index+row])
+                        out[index+row] = data[idx[i]];
+                }
+            }
+        }
+        data.swap(out);
+    }
+    return data;
+}
 
-    MatrixXd out = MatrixXd::Zero(d.rows()+2*r, d.cols()+2*r);
-    out.block(r, r, d.rows(), d.cols()) = d;
-    out.block(r, 0, d.rows(), r) =
-        d.block(0, 0, d.rows(), r).rowwise().reverse();
-    out.block(r, d.cols()+r, d.rows(), r) =
-        d.block(0, d.cols()-r, d.rows(), r).rowwise().reverse();
-    out.block(0, 0, r, out.cols()) =
-        out.block(r, 0, r, out.cols()).colwise().reverse();
-    out.block(d.rows()+r, 0, r, out.cols()) =
-        out.block(out.rows()-r-1, 0, r, out.cols()).colwise().reverse();
-
-    return out;
+std::vector<double> erodeDiamond(std::vector<double> data, size_t rows, size_t cols, int iterations)
+{
+    std::vector<double> out(data.size(), std::numeric_limits<double>::max());
+    std::vector<size_t> idx(5);
+    
+    for (int iter = 0; iter < iterations; ++iter)
+    {
+        for (size_t col = 0; col < cols; ++col)
+        {
+            size_t index = col*rows;
+            for (size_t row = 0; row < rows; ++row)
+            {
+                size_t j = 0;
+                idx[j++] = index+row;
+                if (row > 0)
+                    idx[j++] = idx[0]-1;
+                if (row < rows-1)
+                    idx[j++] = idx[0]+1;
+                if (col > 0)
+                    idx[j++] = idx[0]-rows;
+                if (col < cols-1)
+                    idx[j++] = idx[0]+rows;
+                for (size_t i = 0; i < j; ++i)
+                {
+                    if (data[idx[i]] < out[index+row])
+                        out[index+row] = data[idx[i]];
+                }
+            }
+        }
+        data.swap(out);
+    }
+    return data;
 }
 
 Eigen::MatrixXd pointViewToEigen(const PointView& view)
