@@ -250,6 +250,9 @@ PDAL_DLL uint8_t computeRank(PointView& view, std::vector<PointId> ids,
 PDAL_DLL Eigen::MatrixXd createMaxMatrix(PointView& view, int rows, int cols,
         double cell_size, BOX2D bounds);
 
+PDAL_DLL Eigen::MatrixXd createMaxMatrix2(PointView& view, int rows, int cols,
+        double cell_size, BOX2D bounds);
+
 /**
   Create matrix of minimum Z values.
 
@@ -265,7 +268,7 @@ PDAL_DLL Eigen::MatrixXd createMaxMatrix(PointView& view, int rows, int cols,
 */
 PDAL_DLL Eigen::MatrixXd createMinMatrix(PointView& view, int rows, int cols,
         double cell_size, BOX2D bounds);
-
+        
 /**
   Find local minimum elevations by extended local minimum.
 
@@ -313,6 +316,44 @@ PDAL_DLL Eigen::MatrixXd matrixClose(Eigen::MatrixXd data, int radius);
 */
 PDAL_DLL Eigen::MatrixXd matrixOpen(Eigen::MatrixXd data, int radius);
 
+/**
+  Perform a morphological dilation of the input raster.
+
+  Performs a morphological dilation of the input raster using a diamond
+  structuring element. Larger structuring elements are approximated by applying
+  multiple iterations of the opening operation. The input and output rasters are
+  stored in column major order.
+
+  \param data the input raster.
+  \param rows the number of rows.
+  \param cols the number of cols.
+  \param iterations the number of iterations used to approximate a larger 
+         structuring element.
+  \return the morphological dilation of the input raster.
+*/
+PDAL_DLL std::vector<double> dilateDiamond(std::vector<double> data,
+                                           size_t rows, size_t cols,
+                                           int iterations);
+
+/**
+  Perform a morphological erosion of the input raster.
+
+  Performs a morphological erosion of the input raster using a diamond
+  structuring element. Larger structuring elements are approximated by applying
+  multiple iterations of the opening operation. The input and output rasters are
+  stored in column major order.
+
+  \param data the input raster.
+  \param rows the number of rows.
+  \param cols the number of cols.
+  \param iterations the number of iterations used to approximate a larger 
+         structuring element.
+  \return the morphological erosion of the input raster.
+*/
+PDAL_DLL std::vector<double> erodeDiamond(std::vector<double> data,
+                                          size_t rows, size_t cols,
+                                          int iterations);
+
 /**
   Pad input matrix symmetrically.
 
@@ -322,7 +363,24 @@ PDAL_DLL Eigen::MatrixXd matrixOpen(Eigen::MatrixXd data, int radius);
   \param r the radius of the padding.
   \return the padded matrix.
 */
-PDAL_DLL Eigen::MatrixXd padMatrix(Eigen::MatrixXd d, int r);
+template <typename Derived>
+PDAL_DLL Derived padMatrix(const Eigen::MatrixBase<Derived>& d, int r)
+{
+    using namespace Eigen;
+
+    Derived out = Derived::Zero(d.rows()+2*r, d.cols()+2*r);
+    out.block(r, r, d.rows(), d.cols()) = d;
+    out.block(r, 0, d.rows(), r) =
+        d.block(0, 0, d.rows(), r).rowwise().reverse();
+    out.block(r, d.cols()+r, d.rows(), r) =
+        d.block(0, d.cols()-r, d.rows(), r).rowwise().reverse();
+    out.block(0, 0, r, out.cols()) =
+        out.block(r, 0, r, out.cols()).colwise().reverse();
+    out.block(d.rows()+r, 0, r, out.cols()) =
+        out.block(out.rows()-r-1, 0, r, out.cols()).colwise().reverse();
+
+    return out;
+}
 
 /**
   Converts a PointView into an Eigen::MatrixXd.
@@ -721,12 +779,14 @@ PDAL_DLL double computeSlopeD8(const Eigen::MatrixBase<Derived>& data,
     submatrix.setConstant(data(1, 1));
     submatrix -= data;
     submatrix /= spacing;
-    submatrix(0, 1) /= std::sqrt(2.0);
-    submatrix(1, 0) /= std::sqrt(2.0);
-    submatrix(1, 2) /= std::sqrt(2.0);
-    submatrix(2, 1) /= std::sqrt(2.0);
+    submatrix(0, 0) /= std::sqrt(2.0);
+    submatrix(0, 2) /= std::sqrt(2.0);
+    submatrix(2, 0) /= std::sqrt(2.0);
+    submatrix(2, 2) /= std::sqrt(2.0);
 
-    // find max and convert to degrees
+    // Why not just use Eigen's maxCoeff reduction to find the max? Well, as it
+    // turns out, if there is a chance that we will have NaN's then maxCoeff
+    // has no way to ignore the NaN.
     double maxval = std::numeric_limits<double>::lowest();
     for (int i = 0; i < submatrix.size(); ++i)
     {
@@ -760,6 +820,118 @@ PDAL_DLL double computeSlopeFD(const Eigen::MatrixBase<Derived>& data,
     return 100.0 * std::sqrt(p);
 }
 
+/**
+  Perform a morphological dilation of the input matrix.
+
+  Performs a morphological dilation of the input matrix using a circular
+  structuring element of given radius.
+
+  \param data the input matrix.
+  \param radius the radius of the circular structuring element.
+  \return the morphological dilation of the input matrix.
+*/
+template <typename Derived>
+PDAL_DLL Derived dilate(const Eigen::MatrixBase<Derived>& A, int radius)
+{
+    Derived B = Derived::Constant(A.rows(), A.cols(), 0);
+
+    int length = 2 * radius + 1;
+    bool match_flag;
+    for (int c = 0; c < A.cols(); ++c)
+    {
+        for (int r = 0; r < A.rows(); ++r)
+        {
+            match_flag = false;
+            for (int k = 0; k < length; ++k)
+            {
+                if (match_flag)
+                    break;
+                int cdiff = k-radius;
+                int cpos = c+cdiff;
+                if (cpos < 0 || cpos >= A.cols())
+                    continue;
+                for (int l = 0; l < length; ++l)
+                {
+                    int rdiff = l-radius;
+                    int rpos = r+rdiff;
+                    if (rpos < 0 || rpos >= A.rows())
+                        continue;
+                    if ((cdiff*cdiff+rdiff*rdiff) > radius*radius)
+                        continue;
+                    if (A(rpos, cpos) == 1)
+                    {
+                        match_flag = true;
+                        break;
+                    }
+                }
+            }
+            // Assign value according to match flag
+            B(r, c) = (match_flag) ? 1 : 0;
+        }
+    }
+
+    return B;
+}
+
+/**
+  Perform a morphological erosion of the input matrix.
+
+  Performs a morphological erosion of the input matrix using a circular
+  structuring element of given radius.
+
+  \param data the input matrix.
+  \param radius the radius of the circular structuring element.
+  \return the morphological erosion of the input matrix.
+*/
+template <typename Derived>
+PDAL_DLL Derived erode(const Eigen::MatrixBase<Derived>& A, int radius)
+{
+    Derived B = Derived::Constant(A.rows(), A.cols(), 1);
+
+    int length = 2 * radius + 1;
+    bool mismatch_flag;
+    for (int c = 0; c < A.cols(); ++c)
+    {
+        for (int r = 0; r < A.rows(); ++r)
+        {
+            if (A(r, c) == 0)
+            {
+                B(r, c) = 0;
+                continue;
+            }
+            mismatch_flag = false;
+            for (int k = 0; k < length; k++)
+            {
+                if (mismatch_flag)
+                    break;
+                int cdiff = k-radius;
+                int cpos = c+cdiff;
+                if (cpos < 0 || cpos >= A.cols())
+                    continue;
+                for (int l = 0; l < length; l++)
+                {
+                    int rdiff = l-radius;
+                    int rpos = r+rdiff;
+                    if (rpos < 0 || rpos >= A.rows())
+                        continue;
+                    if ((cdiff*cdiff+rdiff*rdiff) > radius*radius)
+                        continue;
+                    if (A(rpos, cpos) == 0)
+                    {
+                        B(r, c) = 0;
+                        mismatch_flag = true;
+                        break;
+                    }
+                }
+            }
+            // Assign value according to mismatch flag
+            B(r, c) = (mismatch_flag) ? 0 : 1;
+        }
+    }
+
+    return B;
+}
+
 /**
   Thin Plate Spline interpolation.
 
