@@ -66,8 +66,7 @@ namespace
                 const std::string jsonError(reader.getFormattedErrorMessages());
                 if (!jsonError.empty())
                 {
-                    throw std::runtime_error(
-                            "Error during parsing: " + jsonError);
+                    throw pdal_error("Error during parsing: " + jsonError);
                 }
             }
         }
@@ -238,42 +237,20 @@ void GreyhoundReader::initialize(PointTableRef table)
     std::string infoUrl = m_url + "/resource/" + m_resource + "/info";
     log()->get(LogLevel::Debug) << "Fetching info URL: " << infoUrl <<
         std::endl;
-    m_info = parse(m_arbiter->get(infoUrl));
+    try
+    {
+        m_info = parse(m_arbiter->get(infoUrl));
+    }
+    catch (const pdal_error& err)
+    {
+        throwError(err.what());
+    }
 
     m_depthBegin = m_depthBeginArg;
     m_depthEnd = m_depthEndArg;
 
-    if (m_info.isMember("scale"))
-    {
-        m_scale.reset(new greyhound::Point(m_info["scale"]));
-    }
-
-    if (m_info.isMember("offset"))
-    {
-        m_offset.reset(new greyhound::Point(m_info["offset"]));
-    }
-
-    if (m_scale && !m_offset) m_offset.reset(new greyhound::Point(0, 0, 0));
-    if (m_offset && !m_scale) m_scale.reset(new greyhound::Point(1, 1, 1));
-
     m_fullBounds = m_info["bounds"];
 
-    if (m_scale)
-    {
-        // Unscale the full bounds.  Since the query bounds will come in as
-        // native coordinates, don't modify those.
-        m_fullBounds = m_fullBounds.unscale(*m_scale, *m_offset);
-
-        // Now inverse our scale/offset.
-        m_scale->x = 1.0 / m_scale->x;
-        m_scale->y = 1.0 / m_scale->y;
-        m_scale->z = 1.0 / m_scale->z;
-
-        m_offset->x = -m_offset->x;
-        m_offset->y = -m_offset->y;
-        m_offset->z = -m_offset->z;
-    }
-
     m_queryBounds = toBounds(m_queryBox).intersection(m_fullBounds);
 
     if (m_pathsArg.size())
@@ -335,8 +312,7 @@ void GreyhoundReader::prepared(PointTableRef table)
 
         if (m_dims.back().m_id == Dimension::Id::Unknown)
         {
-            throw std::runtime_error(
-                    "Could not find dimension " + j["name"].asString());
+            throwError("Could not find dimension " + j["name"].asString());
         }
     }
 }
@@ -444,7 +420,8 @@ point_count_t GreyhoundReader::read(PointViewPtr view, point_count_t count)
     launchPooledReads(*view, zoomBounds, depthSplit, pool);
 
     pool.await();
-    if (m_error) throw pdal_error(*m_error);
+    if (m_error)
+        throwError(*m_error);
 
     return m_numPoints;
 }
@@ -509,7 +486,8 @@ void GreyhoundReader::launchPooledReads(
 
         // If any tasks failed, rethrow in the main thread.
         lock.lock();
-        if (m_error) throw pdal_error(*m_error);
+        if (m_error)
+            throwError(*m_error);
     }
 }
 
@@ -592,15 +570,20 @@ std::vector<point_count_t> GreyhoundReader::fetchVerticalHierarchy(
     url << "&depthEnd=" << depthEnd;
     url << "&vertical=true";
 
-    if (m_scale) url << "&scale=" << write(m_scale->toJson());
-    if (m_offset) url << "&offset=" << write(m_offset->toJson());
-
     log()->get(LogLevel::Debug) << "Hierarchy: " << url.str() << std::endl;
-    const Json::Value json(parse(m_arbiter->get(url.str())));
 
     std::vector<point_count_t> results;
-    for (const auto& v : json) results.push_back(v.asUInt64());
+    try
+    {
+        const Json::Value json(parse(m_arbiter->get(url.str())));
 
+        for (const auto& v : json)
+            results.push_back(v.asUInt64());
+    }
+    catch (const pdal_error& err)
+    {
+        throwError(err.what());
+    }
     return results;
 }
 
@@ -615,11 +598,18 @@ Json::Value GreyhoundReader::fetchHierarchy(
     url << "&depthBegin=" << depthBegin;
     url << "&depthEnd=" << depthEnd;
 
-    if (m_scale) url << "&scale=" << write(m_scale->toJson());
-    if (m_offset) url << "&offset=" << write(m_offset->toJson());
-
     log()->get(LogLevel::Debug) << "Hierarchy: " << url.str() << std::endl;
-    return parse(m_arbiter->get(url.str()));
+
+    Json::Value json;
+    try
+    {
+        json = parse(m_arbiter->get(url.str()));
+    }
+    catch (const pdal_error& err)
+    {
+        throwError(err.what());
+    }
+    return json;
 }
 
 point_count_t GreyhoundReader::fetchData(
@@ -633,8 +623,6 @@ point_count_t GreyhoundReader::fetchData(
     url << "/read?bounds=" << arbiter::http::sanitize(stringify(bounds));
     url << "&depthBegin=" << depthBegin;
     url << "&depthEnd=" << depthEnd;
-    if (m_scale) url << "&scale=" << write(m_scale->toJson());
-    if (m_offset) url << "&offset=" << write(m_offset->toJson());
 
 #ifdef PDAL_HAVE_LAZPERF
     url << "&compress=true";
