@@ -60,6 +60,7 @@ void PredicateFilter::addArgs(ProgramArgs& args)
     args.add("module", "Python module containing the function to run",
         m_module);
     args.add("function", "Function to call", m_function);
+    args.add("pdalargs", "Dictionary to add to module globals when calling function", m_pdalargs);
 }
 
 
@@ -70,7 +71,7 @@ void PredicateFilter::ready(PointTableRef table)
 
     plang::Environment::get()->set_stdout(log()->getLogStream());
     m_script = new plang::Script(m_source, m_module, m_function);
-    m_pythonMethod = new plang::BufferedInvocation(*m_script);
+    m_pythonMethod = new plang::Invocation(*m_script);
     m_pythonMethod->compile();
 }
 
@@ -81,11 +82,18 @@ PointViewSet PredicateFilter::run(PointViewPtr view)
 
     m_pythonMethod->resetArguments();
     m_pythonMethod->begin(*view, n);
+
+    if (!m_pdalargs.empty())
+    {
+        std::ostringstream args;
+        args << m_pdalargs;
+        m_pythonMethod->setKWargs(args.str());
+    }
+
     m_pythonMethod->execute();
 
     if (!m_pythonMethod->hasOutputVariable("Mask"))
-        throw pdal::pdal_error("Mask variable not set in predicate "
-            "filter function.");
+        throwError("Mask variable not set in filter function.");
 
     PointViewPtr outview = view->makeNew();
 
