@@ -37,7 +37,9 @@
 #include <pdal/pdal_internal.hpp>
 #include <pdal/Filter.hpp>
 
-#include <pdal/plang/BufferedInvocation.hpp>
+#include <pdal/plang/Invocation.hpp>
+
+#include <json/json.h>
 
 namespace pdal
 {
@@ -53,12 +55,13 @@ public:
     std::string getName() const;
 
 private:
-    plang::BufferedInvocation* m_pythonMethod;
+    plang::Invocation* m_pythonMethod;
     plang::Script* m_script;
     std::string m_source;
     std::string m_scriptFile;
     std::string m_module;
     std::string m_function;
+    Json::Value m_pdalargs;
 
     virtual void addArgs(ProgramArgs& args);
     virtual void ready(PointTableRef table);
