@@ -37,7 +37,9 @@
 #include <pdal/pdal_internal.hpp>
 #include <pdal/Filter.hpp>
 
-#include <pdal/plang/BufferedInvocation.hpp>
+#include <pdal/plang/Invocation.hpp>
+
+#include <json/json.h>
 
 namespace pdal
 {
@@ -56,7 +58,7 @@ public:
 
 private:
     plang::Script* m_script;
-    plang::BufferedInvocation *m_pythonMethod;
+    plang::Invocation *m_pythonMethod;
     std::string m_source;
     std::string m_scriptFile;
     std::string m_module;
@@ -73,6 +75,7 @@ private:
     ProgrammableFilter(const ProgrammableFilter&); // not implemented
 
     MetadataNode m_totalMetadata;
+    Json::Value m_pdalargs;
 };
 
 } // namespace pdal
