@@ -31,7 +31,6 @@ cmake \
     -DBUILD_PLUGIN_MRSID=OFF \
     -DBUILD_PLUGIN_NITF=OFF \
     -DBUILD_PLUGIN_OCI=OFF \
-    -DBUILD_PLUGIN_P2G=$OPTIONAL_COMPONENT_SWITCH \
     -DBUILD_PLUGIN_PCL=$OPTIONAL_COMPONENT_SWITCH \
     -DBUILD_PLUGIN_PGPOINTCLOUD=$OPTIONAL_COMPONENT_SWITCH \
     -DBUILD_PGPOINTCLOUD_TESTS=OFF \
@@ -43,8 +42,6 @@ cmake \
     -DWITH_LAZPERF=$OPTIONAL_COMPONENT_SWITCH \
     -DWITH_LASZIP=$OPTIONAL_COMPONENT_SWITCH \
     -DWITH_PDAL_JNI=$OPTIONAL_COMPONENT_SWITCH \
-    -DLASZIP_INCLUDE_DIR:PATH=/usr/include \
-    -DLASZIP_LIBRARY:FILEPATH=/usr/lib/liblaszip.so \
     -DWITH_TESTS=ON \
     -G "$PDAL_CMAKE_GENERATOR" \
     ..
