@@ -38,6 +38,7 @@
 #include <pdal/StageFactory.hpp>
 #include <pdal/util/FileUtils.hpp>
 #include <pdal/util/Utils.hpp>
+#include <io/LasReader.hpp>
 #include "Support.hpp"
 
 #include <iostream>
@@ -106,10 +107,30 @@ TEST(pipelineBaseTest, common_opts)
     const std::string cmd = appName();
 
     std::string output;
-    int stat = pdal::Utils::run_shell_command(cmd + " -h", output);
+    int stat = Utils::run_shell_command(cmd + " -h", output);
     EXPECT_EQ(stat, 0);
 }
 
+
+TEST(pipelineBaseTest, progress)
+{
+    std::string cmd = appName();
+    std::string progressOut = Support::temppath("progress.out");
+    FileUtils::deleteFile(progressOut);
+    auto handle = FileUtils::createFile(progressOut);
+    FileUtils::closeFile(handle);
+
+    cmd += " --progress " + progressOut + " "  +
+        Support::configuredpath("pipeline/bpf2las.json");
+
+    std::string output;
+    EXPECT_EQ(Utils::run_shell_command(cmd, output), 0);
+
+    std::string progress = FileUtils::readFileIntoString(progressOut);
+    EXPECT_NE(progress.find("READYFILE"), std::string::npos);
+    EXPECT_NE(progress.find("DONEFILE"), std::string::npos);
+}
+
 class json : public testing::TestWithParam<const char*> {};
 
 TEST_P(json, pipeline)
@@ -126,7 +147,7 @@ TEST(json, pipeline_stdin)
 INSTANTIATE_TEST_CASE_P(base, json,
                         testing::Values(
                             // "autzen/autzen-interpolate.json",
-                            "pipeline/attribute.json",
+                            "pipeline/assign.json",
                             "pipeline/bpf2las.json",
                             "pipeline/chipper.json",
                             "pipeline/colorize-multi.json",
@@ -144,6 +165,7 @@ INSTANTIATE_TEST_CASE_P(base, json,
                             "pipeline/metadata_writer.json",
                             "pipeline/mississippi.json",
                             "pipeline/mississippi_reverse.json",
+                            "pipeline/overlay.json",
                             // "pipeline/qfit2las.json",
                             "pipeline/range_z.json",
                             "pipeline/range_z_classification.json",
@@ -258,4 +280,80 @@ TEST(json, issue1417)
     run_pipeline("pipeline/issue1417.json", options);
 }
 
+// Test that stage options passed via --stage.<tagname>.<option> work.
+TEST(json, stagetags)
+{
+    auto checkValue = [](const std::string filename, double value)
+    {
+        Options o;
+        o.add("filename", filename);
+        LasReader r;
+        r.setOptions(o);
+
+        PointTable t;
+        r.prepare(t);
+        PointViewSet s = r.execute(t);
+        EXPECT_EQ(s.size(), 1u);
+        PointViewPtr v = *s.begin();
+
+        for (PointId i = 0; i < v->size(); ++i)
+            EXPECT_DOUBLE_EQ(value,
+                v->getFieldAs<double>(Dimension::Id::Z, i));
+
+        FileUtils::deleteFile(filename);
+    };
+
+    std::string outFilename(Support::temppath("assigned.las"));
+    std::string base(appName() + " " +
+        Support::configuredpath("pipeline/options.json"));
+    std::string output;
+    int stat;
+
+    stat = Utils::run_shell_command(base, output);
+    EXPECT_EQ(stat, 0);
+    checkValue(outFilename, 25);
+
+    stat = Utils::run_shell_command(base +
+        " --filters.assign.assignment=Z[:]=101",
+        output);
+    EXPECT_EQ(stat, 0);
+    checkValue(outFilename, 101);
+
+    stat = Utils::run_shell_command(base +
+        " --stage.assigner.assignment=Z[:]=1987",
+        output);
+    EXPECT_EQ(stat, 0);
+    checkValue(outFilename, 1987);
+
+    // Make sure that tag options override stage options.
+    stat = Utils::run_shell_command(base +
+        " --filters.assign.assignment=Z[:]=25 "
+        "--stage.assigner.assignment=Z[:]=555", output);
+    EXPECT_EQ(stat, 0);
+    checkValue(outFilename, 555);
+    stat = Utils::run_shell_command(base +
+        " --stage.assigner.assignment=Z[:]=555 "
+        "--filters.assign.assignment=Z[:]=25 ", output);
+    EXPECT_EQ(stat, 0);
+    checkValue(outFilename, 555);
+
+    // Check that bad tag fails.
+    stat = Utils::run_shell_command(base +
+        " --stage.foobar.assignment=Z[:]=1987",
+        output);
+    EXPECT_NE(stat, 0);
+
+    // Check that bad option name fails.
+    stat = Utils::run_shell_command(base +
+        " --stage.assigner.blah=Z[:]=1987",
+        output);
+    EXPECT_NE(stat, 0);
+
+    // Check that multiply specified option fails.
+    stat = Utils::run_shell_command(base +
+        " --stage.reader.compression=laszip "
+        "--stage.reader.compression=lazperf", output);
+    EXPECT_NE(stat, 0);
+}
+
 } // namespace pdal
