@@ -59,6 +59,9 @@ public:
         { return &w.m_lasHeader; }
     SpatialReference srs(LasWriter& w)
         { return w.m_srs; }
+    void addVlr(LasWriter& w, const std::string& userId, uint16_t recordId,
+        std::string description, std::vector<uint8_t>& data)
+        { w.addVlr(userId, recordId, description, data); }
 };
 
 } // namespace pdal
@@ -653,6 +656,29 @@ TEST(LasWriterTest, stream)
     compareFiles(infile, outfile);
 }
 
+TEST(LasWriterTest, streamhashwrite)
+{
+    std::string infile(Support::datapath("las/autzen_trim.las"));
+    std::string outfile(Support::temppath("trimtest#.las"));
+
+    FileUtils::deleteFile(outfile);
+
+    Options ops1;
+    ops1.add("filename", infile);
+
+    LasReader r;
+    r.setOptions(ops1);
+
+    Options ops2;
+    ops2.add("filename", outfile);
+    LasWriter w;
+    w.setOptions(ops2);
+    w.setInput(r);
+
+    FixedPointTable t(100);
+    EXPECT_THROW(w.prepare(t), pdal_error);
+}
+
 TEST(LasWriterTest, fix1063_1064_1065)
 {
     std::string outfile = Support::temppath("out.las");
@@ -663,8 +689,6 @@ TEST(LasWriterTest, fix1063_1064_1065)
     std::string cmd = "pdal translate --writers.las.forward=all "
         "--writers.las.a_srs=\"EPSG:4326\" " + infile + " " + outfile;
     std::string output;
-    std::cerr << "*** Shell command = " <<
-        Support::binpath(cmd) << "!\n";
     Utils::run_shell_command(Support::binpath(cmd), output);
 
     Options o;
@@ -695,7 +719,175 @@ TEST(LasWriterTest, fix1063_1064_1065)
     EXPECT_EQ(ref.getWKT(), wkt);
 }
 
+TEST(LasWriterTest, pdal_metadata)
+{
+    PointTable table;
+
+    std::string infile(Support::datapath("las/1.2-with-color.las"));
+    std::string outfile(Support::temppath("simple.las"));
+
+    // remove file from earlier run, if needed
+    FileUtils::deleteFile(outfile);
+
+    Options readerOpts;
+    readerOpts.add("filename", infile);
+
+    Options writerOpts;
+    writerOpts.add("pdal_metadata", true);
+    writerOpts.add("filename", outfile);
+
+    LasReader reader;
+    reader.setOptions(readerOpts);
+
+    LasWriter writer;
+    writer.setOptions(writerOpts);
+    writer.setInput(reader);
+    writer.prepare(table);
+    writer.execute(table);
+
+    PointTable t2;
+    Options readerOpts2;
+    readerOpts2.add("filename", outfile);
+    LasReader reader2;
+    reader2.setOptions(readerOpts2);
+
+    reader2.prepare(t2);
+    reader2.execute(t2);
+
+    EXPECT_EQ(reader2.getMetadata().children("pdal_metadata").size(), 1UL);
+    EXPECT_EQ(reader2.getMetadata().children("pdal_pipeline").size(), 1UL);
+
+}
+
+
+TEST(LasWriterTest, pdal_add_vlr)
+{
+    PointTable table;
+
+    std::string infile(Support::datapath("las/1.2-with-color.las"));
+    std::string outfile(Support::temppath("simple.las"));
+
+    // remove file from earlier run, if needed
+    FileUtils::deleteFile(outfile);
+
+    Options readerOpts;
+    readerOpts.add("filename", infile);
+
+    std::string vlr( " [ { \"description\": \"A description under 32 bytes\", \"record_id\": 42, \"user_id\": \"hobu\", \"data\": \"dGhpcyBpcyBzb21lIHRleHQ=\" },  { \"description\": \"A description under 32 bytes\", \"record_id\": 43, \"user_id\": \"hobu\", \"data\": \"dGhpcyBpcyBzb21lIG1vcmUgdGV4dA==\" } ]");
+
+    Options writerOpts;
+    writerOpts.add("vlrs", vlr);
+    writerOpts.add("filename", outfile);
+
+    LasReader reader;
+    reader.setOptions(readerOpts);
+
+    LasWriter writer;
+    writer.setOptions(writerOpts);
+    writer.setInput(reader);
+    writer.prepare(table);
+    writer.execute(table);
+
+    PointTable t2;
+    Options readerOpts2;
+    readerOpts2.add("filename", outfile);
+    LasReader reader2;
+    reader2.setOptions(readerOpts2);
+
+    reader2.prepare(t2);
+    reader2.execute(t2);
+
+    MetadataNode forward = reader2.getMetadata();
+
+    auto pred = [](MetadataNode temp)
+        { return Utils::startsWith(temp.name(), "vlr_"); };
+    MetadataNodeList nodes = forward.findChildren(pred);
+    EXPECT_EQ(nodes.size(), 2UL);
+}
+
+
+// Make sure that we can forward the LAS_Spec/3 VLR
+TEST(LasWriterTest, forward_spec_3)
+{
+    PointTable table;
+
+    std::string infile(Support::datapath("las/spec_3.las"));
+    std::string outfile(Support::temppath("out.las"));
+
+    // remove file from earlier run, if needed
+    FileUtils::deleteFile(outfile);
+
+    Options readerOpts;
+    readerOpts.add("filename", infile);
+
+    Options writerOpts;
+    writerOpts.add("forward", "all,vlr");
+    writerOpts.add("filename", outfile);
+
+    LasReader reader;
+    reader.setOptions(readerOpts);
+
+    LasWriter writer;
+    writer.setOptions(writerOpts);
+    writer.setInput(reader);
+
+    writer.prepare(table);
+    writer.execute(table);
+
+    PointTable t2;
+    Options readerOpts2;
+    readerOpts2.add("filename", outfile);
+    LasReader reader2;
+    reader2.setOptions(readerOpts2);
+
+    reader2.prepare(t2);
+    reader2.execute(t2);
+
+    auto pred = [](MetadataNode temp)
+    {
+        auto recPred = [](MetadataNode n)
+        {
+            return n.name() == "record_id" &&
+                n.value() == "3";
+        };
+
+        auto userPred = [](MetadataNode n)
+        {
+            return n.name() == "user_id" &&
+                n.value() == "LASF_Spec";
+        };
+
+        return Utils::startsWith(temp.name(), "vlr_") &&
+            !temp.findChild(recPred).empty() &&
+            !temp.findChild(userPred).empty();
+    };
+    MetadataNode root = reader2.getMetadata();
+    MetadataNodeList nodes = root.findChildren(pred);
+    EXPECT_EQ(nodes.size(), 1u);
+}
+
+TEST(LasWriterTest, oversize_vlr)
+{
+    LasWriter w;
+    Options o;
+
+    o.add("filename", "out.las");
+    w.addOptions(o);
+
+    PointTable t;
+
+    w.prepare(t);
+
+    std::vector<uint8_t> data(100000, 32);
+    LasTester tester;
+    EXPECT_THROW(
+        tester.addVlr(w, "USER ID", 555, "This is a description", data),
+        pdal_error);
+}
+
+
 /**
+
 namespace
 {
 
